"""The ref commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - REF<x>:DATE?
    - REF<x>:HORizontal:DELay:TIMe <NR3>
    - REF<x>:HORizontal:DELay:TIMe?
    - REF<x>:HORizontal:SCAle <NR3>
    - REF<x>:HORizontal:SCAle?
    - REF<x>:LABel <Qstring>
    - REF<x>:LABel?
    - REF<x>:TIMe?
    - REF<x>:VERTical:POSition <NR3>
    - REF<x>:VERTical:POSition?
    - REF<x>:VERTical:SCAle <NR3>
    - REF<x>:VERTical:SCAle?
    - REF<x>?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class RefItemVerticalScale(SCPICmdWrite, SCPICmdRead):
    """The ``REF<x>:VERTical:SCAle`` command.

    **Description:**
        - This command specifies the vertical scale for the reference waveform specified by <x>.
          This setting controls the display only. For a signal with constant amplitude, increasing
          the scale causes the waveform to be displayed smaller. Decreasing the scale causes the
          waveform to be displayed larger.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:VERTical:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF<x>:VERTical:SCAle value`` command.

    **SCPI Syntax:**

    ::

        - REF<x>:VERTical:SCAle <NR3>
        - REF<x>:VERTical:SCAle?

    **Info:**
        - ``<NR3>`` is a floating point number that specifies the gain in user units-per-division.
    """


class RefItemVerticalPosition(SCPICmdWrite, SCPICmdRead):
    """The ``REF<x>:VERTical:POSition`` command.

    **Description:**
        - This command specifies the vertical position of the reference waveform specified by <x>.
          Increasing the position value of a waveform causes the waveform to move up, and decreasing
          the position value causes the waveform to move down. Position adjusts only the display
          position of a waveform. The position value determines the vertical graticule coordinate at
          which signal values are displayed. For example, if the position for Reference 3 is set to
          2.0, the signal represented by that reference will be displayed at 2.0 divisions above the
          center of the screen.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:VERTical:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical:POSition?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF<x>:VERTical:POSition value``
          command.

    **SCPI Syntax:**

    ::

        - REF<x>:VERTical:POSition <NR3>
        - REF<x>:VERTical:POSition?

    **Info:**
        - ``<NR3>`` is a floating point number that specifies the desired position, in divisions
          from the center horizontal graticule. The range is from -5.0 to 5.0 divisions.
    """


class RefItemVertical(SCPICmdRead):
    """The ``REF<x>:VERTical`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:VERTical?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``REF<x>:VERTical:POSition`` command.
        - ``.scale``: The ``REF<x>:VERTical:SCAle`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = RefItemVerticalPosition(device, f"{self._cmd_syntax}:POSition")
        self._scale = RefItemVerticalScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def position(self) -> RefItemVerticalPosition:
        """Return the ``REF<x>:VERTical:POSition`` command.

        **Description:**
            - This command specifies the vertical position of the reference waveform specified by
              <x>. Increasing the position value of a waveform causes the waveform to move up, and
              decreasing the position value causes the waveform to move down. Position adjusts only
              the display position of a waveform. The position value determines the vertical
              graticule coordinate at which signal values are displayed. For example, if the
              position for Reference 3 is set to 2.0, the signal represented by that reference will
              be displayed at 2.0 divisions above the center of the screen.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:VERTical:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical:POSition?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF<x>:VERTical:POSition value``
              command.

        **SCPI Syntax:**

        ::

            - REF<x>:VERTical:POSition <NR3>
            - REF<x>:VERTical:POSition?

        **Info:**
            - ``<NR3>`` is a floating point number that specifies the desired position, in divisions
              from the center horizontal graticule. The range is from -5.0 to 5.0 divisions.
        """
        return self._position

    @property
    def scale(self) -> RefItemVerticalScale:
        """Return the ``REF<x>:VERTical:SCAle`` command.

        **Description:**
            - This command specifies the vertical scale for the reference waveform specified by <x>.
              This setting controls the display only. For a signal with constant amplitude,
              increasing the scale causes the waveform to be displayed smaller. Decreasing the scale
              causes the waveform to be displayed larger.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:VERTical:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical:SCAle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF<x>:VERTical:SCAle value``
              command.

        **SCPI Syntax:**

        ::

            - REF<x>:VERTical:SCAle <NR3>
            - REF<x>:VERTical:SCAle?

        **Info:**
            - ``<NR3>`` is a floating point number that specifies the gain in user
              units-per-division.
        """
        return self._scale


class RefItemTime(SCPICmdRead):
    """The ``REF<x>:TIMe`` command.

    **Description:**
        - Returns the time that reference waveform data was copied into the internal reference
          memory for reference waveform <x>.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:TIMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - REF<x>:TIMe?
    """


class RefItemLabel(SCPICmdWrite, SCPICmdRead):
    """The ``REF<x>:LABel`` command.

    **Description:**
        - This command specifies the label for the reference waveform specified by <x>.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF<x>:LABel value`` command.

    **SCPI Syntax:**

    ::

        - REF<x>:LABel <Qstring>
        - REF<x>:LABel?

    **Info:**
        - ``<Qstring>`` is an alphanumeric string of text, enclosed in quotes, that contains the
          label text for the reference channel <x> waveform. The text string is limited to 30
          characters.
    """


class RefItemHorizontalScale(SCPICmdWrite, SCPICmdRead):
    """The ``REF<x>:HORizontal:SCAle`` command.

    **Description:**
        - This command specifies the horizontal scale for the reference waveform specified by <x>.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:HORizontal:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF<x>:HORizontal:SCAle value``
          command.

    **SCPI Syntax:**

    ::

        - REF<x>:HORizontal:SCAle <NR3>
        - REF<x>:HORizontal:SCAle?

    **Info:**
        - ``<NR3>`` is a floating point number that specifies the horizontal scale in seconds.
    """


class RefItemHorizontalDelayTime(SCPICmdWrite, SCPICmdRead):
    """The ``REF<x>:HORizontal:DELay:TIMe`` command.

    **Description:**
        - This command specifies the horizontal delay time for reference waveform <x>. The delay
          time is expressed in seconds and is limited to ±5 times the reference horizontal scale.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:HORizontal:DELay:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:DELay:TIMe?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF<x>:HORizontal:DELay:TIMe value``
          command.

    **SCPI Syntax:**

    ::

        - REF<x>:HORizontal:DELay:TIMe <NR3>
        - REF<x>:HORizontal:DELay:TIMe?

    **Info:**
        - ``<NR3>`` is a floating point number that specifies the delay time, in seconds.
    """


class RefItemHorizontalDelay(SCPICmdRead):
    """The ``REF<x>:HORizontal:DELay`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:HORizontal:DELay?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:DELay?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.time``: The ``REF<x>:HORizontal:DELay:TIMe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._time = RefItemHorizontalDelayTime(device, f"{self._cmd_syntax}:TIMe")

    @property
    def time(self) -> RefItemHorizontalDelayTime:
        """Return the ``REF<x>:HORizontal:DELay:TIMe`` command.

        **Description:**
            - This command specifies the horizontal delay time for reference waveform <x>. The delay
              time is expressed in seconds and is limited to ±5 times the reference horizontal
              scale.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:HORizontal:DELay:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:DELay:TIMe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``REF<x>:HORizontal:DELay:TIMe value`` command.

        **SCPI Syntax:**

        ::

            - REF<x>:HORizontal:DELay:TIMe <NR3>
            - REF<x>:HORizontal:DELay:TIMe?

        **Info:**
            - ``<NR3>`` is a floating point number that specifies the delay time, in seconds.
        """
        return self._time


class RefItemHorizontal(SCPICmdRead):
    """The ``REF<x>:HORizontal`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:HORizontal?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.delay``: The ``REF<x>:HORizontal:DELay`` command tree.
        - ``.scale``: The ``REF<x>:HORizontal:SCAle`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._delay = RefItemHorizontalDelay(device, f"{self._cmd_syntax}:DELay")
        self._scale = RefItemHorizontalScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def delay(self) -> RefItemHorizontalDelay:
        """Return the ``REF<x>:HORizontal:DELay`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:HORizontal:DELay?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:DELay?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.time``: The ``REF<x>:HORizontal:DELay:TIMe`` command.
        """
        return self._delay

    @property
    def scale(self) -> RefItemHorizontalScale:
        """Return the ``REF<x>:HORizontal:SCAle`` command.

        **Description:**
            - This command specifies the horizontal scale for the reference waveform specified by
              <x>.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:HORizontal:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal:SCAle?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF<x>:HORizontal:SCAle value``
              command.

        **SCPI Syntax:**

        ::

            - REF<x>:HORizontal:SCAle <NR3>
            - REF<x>:HORizontal:SCAle?

        **Info:**
            - ``<NR3>`` is a floating point number that specifies the horizontal scale in seconds.
        """
        return self._scale


class RefItemDate(SCPICmdRead):
    """The ``REF<x>:DATE`` command.

    **Description:**
        - Returns the date that reference waveform data for reference waveform <x> was copied into
          the internal reference memory.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>:DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>:DATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - REF<x>:DATE?
    """


class RefItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``REF<x>`` command.

    **Description:**
        - This query returns data for the reference waveform specified by <x>.

    **Usage:**
        - Using the ``.query()`` method will send the ``REF<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``REF<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - REF<x>?

    Properties:
        - ``.date``: The ``REF<x>:DATE`` command.
        - ``.horizontal``: The ``REF<x>:HORizontal`` command tree.
        - ``.label``: The ``REF<x>:LABel`` command.
        - ``.time``: The ``REF<x>:TIMe`` command.
        - ``.vertical``: The ``REF<x>:VERTical`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "REF<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._date = RefItemDate(device, f"{self._cmd_syntax}:DATE")
        self._horizontal = RefItemHorizontal(device, f"{self._cmd_syntax}:HORizontal")
        self._label = RefItemLabel(device, f"{self._cmd_syntax}:LABel")
        self._time = RefItemTime(device, f"{self._cmd_syntax}:TIMe")
        self._vertical = RefItemVertical(device, f"{self._cmd_syntax}:VERTical")

    @property
    def date(self) -> RefItemDate:
        """Return the ``REF<x>:DATE`` command.

        **Description:**
            - Returns the date that reference waveform data for reference waveform <x> was copied
              into the internal reference memory.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:DATE?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:DATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - REF<x>:DATE?
        """
        return self._date

    @property
    def horizontal(self) -> RefItemHorizontal:
        """Return the ``REF<x>:HORizontal`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:HORizontal?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:HORizontal?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.delay``: The ``REF<x>:HORizontal:DELay`` command tree.
            - ``.scale``: The ``REF<x>:HORizontal:SCAle`` command.
        """
        return self._horizontal

    @property
    def label(self) -> RefItemLabel:
        """Return the ``REF<x>:LABel`` command.

        **Description:**
            - This command specifies the label for the reference waveform specified by <x>.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:LABel?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF<x>:LABel value`` command.

        **SCPI Syntax:**

        ::

            - REF<x>:LABel <Qstring>
            - REF<x>:LABel?

        **Info:**
            - ``<Qstring>`` is an alphanumeric string of text, enclosed in quotes, that contains the
              label text for the reference channel <x> waveform. The text string is limited to 30
              characters.
        """
        return self._label

    @property
    def time(self) -> RefItemTime:
        """Return the ``REF<x>:TIMe`` command.

        **Description:**
            - Returns the time that reference waveform data was copied into the internal reference
              memory for reference waveform <x>.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:TIMe?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - REF<x>:TIMe?
        """
        return self._time

    @property
    def vertical(self) -> RefItemVertical:
        """Return the ``REF<x>:VERTical`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``REF<x>:VERTical?`` query.
            - Using the ``.verify(value)`` method will send the ``REF<x>:VERTical?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``REF<x>:VERTical:POSition`` command.
            - ``.scale``: The ``REF<x>:VERTical:SCAle`` command.
        """
        return self._vertical



"""MSO70KDX device driver module."""
from tm_devices.commands import MSO70KDXMixin
from tm_devices.drivers.pi.scopes.tekscope_5k_7k_70k.mso70k import MSO70K


class MSO70KDX(MSO70KDXMixin, MSO70K):
    """MSO70KDX device driver."""



"""The hdr commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - HDR {<NR1>|OFF|ON}
    - HDR?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Hdr(SCPICmdWrite, SCPICmdRead):
    """The ``HDR`` command.

    **Description:**
        - This command is identical to the HEADer query and is included for backward compatibility
          purposes.

    **Usage:**
        - Using the ``.query()`` method will send the ``HDR?`` query.
        - Using the ``.verify(value)`` method will send the ``HDR?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HDR value`` command.

    **SCPI Syntax:**

    ::

        - HDR {<NR1>|OFF|ON}
        - HDR?

    **Info:**
        - ``<NR1>`` = 0 sets the Response Header Enable State to false; any other value sets this
          state to true, which causes the instrument to send headers on query responses.
        - ``OFF`` sets the Response Header Enable State to false. This causes the instrument to omit
          headers on query responses, so that only the argument is returned.
        - ``ON`` sets the Response Header Enable State to true. This causes the instrument to
          include headers on applicable query responses. You can then use the query response as a
          command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "HDR") -> None:
        super().__init__(device, cmd_syntax)



"""Module containing helpers for the tm_devices package."""
import contextlib
import datetime
import os
import platform
import re
import shlex
import socket
import subprocess  # nosec
import time
import warnings

from enum import EnumMeta
from typing import Any, Dict, Optional, Tuple, Type

from check4updates import check_and_prompt  # type: ignore
from packaging.version import InvalidVersion, Version

from tm_devices.helpers.constants_and_dataclasses import (
    ConnectionTypes,
    DeviceConfigEntry,
    PACKAGE_NAME,
    PYVISA_PY_BACKEND,
    USB_MODEL_ID_LOOKUP,
    VISA_RESOURCE_EXPRESSION_REGEX,
)
from tm_devices.helpers.enums import CustomStrEnum, SupportedModels
from tm_devices.helpers.standalone_functions import validate_address

with warnings.catch_warnings():
    warnings.simplefilter("ignore", UserWarning)
    import pyvisa as visa

    from gpib_ctypes import make_default_gpib  # type: ignore
    from pyvisa import util as pyvisa_util
    from pyvisa.resources import MessageBasedResource

    make_default_gpib()

####################################################################################################
# Private Constants
####################################################################################################
_VISA_SYSTEM_DETAILS: Dict[str, Any] = pyvisa_util.get_system_details()
_KEITHLEY_2_CHAR_MODEL_LOOKUP = {
    "24": "SMU",
    "26": "SMU",
    "22": "PSU",
    "75": "DMM",
    "64": "SMU",
    "65": "SMU",
    "37": "SS",
}


####################################################################################################
# Public Functions
####################################################################################################
def check_for_update(package_name: str = PACKAGE_NAME) -> None:
    """Check for an update for the provided package.

    Args:
        package_name: Check for an update for the provided package.
    """
    starting_dir = os.getcwd()
    try:
        # Check for package updates, set the interval to zero to always check.
        with contextlib.redirect_stdout(None), contextlib.redirect_stderr(None):
            result = check_and_prompt(
                package_name, remind_delay=0, online_check_interval=0, mock_user_input="2"
            )
        if result.installed_version != result.pypi_version:
            print(
                f"\n\n\033[91mVersion {result.pypi_version} of "
                f"{package_name} is available on PyPI.\n"
                f"Version {result.installed_version} of "
                f"{package_name} is currently installed.\n\n"
                f"To upgrade {package_name} run the following command: "
                f"python -m pip install -U {package_name}\n\n\033[0m"
            )
    except FileNotFoundError:
        print(
            f"\n\n\033[91m{package_name} is not installed, "
            f"unable to check for updates.\n\n\033[0m"
        )
    except IndexError:
        print(
            f"\n\n\033[91m{package_name} is not available on PyPI, "
            f"unable to check for updates.\n\n\033[0m"
        )
    finally:
        # Reset the current working directory, the third-party package call changes it.
        os.chdir(starting_dir)


def check_network_connection(
    device_name: str, ip_address: str, verbose: bool = True
) -> Tuple[bool, str]:
    """Check the network connection to the device using the external ping command.

    Args:
        device_name: The name of the device.
        ip_address: The ip address of the device.
        verbose: Set this to False in order to disable printouts.

    Returns:
        A boolean indicating if there is a network connection and
        a string with the result of the ping command.
    """
    if verbose:
        print_with_timestamp(f"({device_name}) ping >> {ip_address}")
        print(f"{get_timestamp_string()} - ")
    ping_result = ping_address(ip_address)
    if verbose:
        print_with_timestamp("Response from ping >>")
        for line in ping_result.strip().split("\n"):
            print(f"    {line}")

    return "ttl=" in ping_result.lower(), ping_result


def check_port_connection(
    device_name: str, ip_address: str, port: int, timeout_seconds: int = 5, verbose: bool = True
) -> bool:
    """Check if the given port is open on the device.

    Args:
        device_name: The name of the device.
        ip_address: The ip address.
        port: The port to check.
        timeout_seconds: The number of seconds to use as the socket timeout.
        verbose: Set this to False in order to disable printouts.

    Returns:
        A boolean indicating if the port is open.
    """
    if verbose:
        print_with_timestamp(f"({device_name}) >> checking if port {port} is open on {ip_address}")

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as temp_socket:
        try:
            temp_socket.settimeout(timeout_seconds)
            temp_socket.connect((ip_address, port))
            temp_socket.shutdown(socket.SHUT_RDWR)
            port_open = True
        except (socket.error, socket.gaierror, socket.herror):
            port_open = False

    if verbose:
        print_with_timestamp(f"Result >> {port_open}")
    return port_open


def check_visa_connection(
    config_entry: DeviceConfigEntry, visa_library: str, device_name: str, verbose: bool = True
) -> bool:
    """Check if a VISA connection can be made to the device.

    Args:
        config_entry: The device listed in the config of the framework to check.
        visa_library: Indicates which visa library to use for the connection.
        device_name: The name of the device.
        verbose: Set this to False in order to disable printouts.

    Returns:
        a boolean indicating if the visa connection was successful
    """
    if verbose:
        print_with_timestamp(
            f"({device_name}) >> checking if a VISA connection can be made to "
            f"{config_entry.get_visa_resource_expression()}"
        )
    try:
        # Create new VISA connection
        temp_resource = create_visa_connection(
            config_entry,
            visa_library=visa_library,
        )
        temp_resource.close()
        visa_connected = True
    except ConnectionError:  # raised by the create_visa_connection() function
        visa_connected = False

    if verbose:
        print_with_timestamp(f"Result >> {visa_connected}")
    return visa_connected


def create_visa_connection(
    device_config_entry: DeviceConfigEntry,
    visa_library: str,
    *,
    retry_connection: bool = False,
) -> MessageBasedResource:
    """Create a VISA resource.

    Args:
        device_config_entry: The device config entry.
        visa_library: A string containing the VISA library to use to create a ResourceManager.
        retry_connection: Boolean indicating if a second connection attempt should be made. If True,
            two attempts are made to establish a VISA connection, with a 60-second delay in between
            each attempt.

    Returns:
        A VISA resource that can be passed into the device driver.

    Raises:
        ConnectionError: The VISA connection couldn't be created to the device.
    """
    # Set up the VISA resource that will be used for communicating with the VISA device
    resource_expression = device_config_entry.get_visa_resource_expression()
    try:
        # noinspection PyTypeChecker
        visa_object: MessageBasedResource = visa.ResourceManager(  # type: ignore
            visa_library
        ).open_resource(resource_expression)
        # Print a warning if PyVISA-py is used when the user didn't specify STANDALONE
        if (
            visa_object.visalib.library_path.path == "py" and visa_library != PYVISA_PY_BACKEND
        ):  # pragma: no cover
            warnings.warn(
                "No VISA installation was detected, defaulting to using "
                "PyVISA-py as the VISA backend.\n\n"
                'Add the "STANDALONE" option to the configuration to silence this warning.',
                stacklevel=4,
            )
    # The broad except is because pyvisa_py can throw a base exception in the tcpip.py file
    except Exception as error_1:  # noqa: BLE001
        if not retry_connection:
            message = f"Unable to establish a VISA connection to {resource_expression}"
            raise ConnectionError(message) from error_1
        time.sleep(60)  # wait 60 seconds and try again
        try:
            # noinspection PyTypeChecker
            visa_object: MessageBasedResource = visa.ResourceManager(  # type: ignore
                visa_library
            ).open_resource(resource_expression)
        # The broad except is because pyvisa_py can throw a base exception in the tcpip.py file
        except Exception as error_2:  # noqa: BLE001
            message = f"Unable to establish a VISA connection to {resource_expression}\n\n"
            if device_config_entry.connection_type in {
                ConnectionTypes.TCPIP,
                ConnectionTypes.SOCKET,
            }:
                message += (
                    f"This is the current ping output for the device at "
                    f"{device_config_entry.address}:\n{ping_address(device_config_entry.address)}"
                )
            raise ConnectionError(message) from error_2

    return _configure_visa_object(visa_object, device_config_entry, visa_library)


def detect_visa_resource_expression(input_str: str) -> Optional[Tuple[str, str]]:
    """Check if a given string is a VISA resource expression.

    This function will check if a string is a VISA resource expression and pull out the pieces
    needed to make a connection using the DeviceManager.

    The pieces consist of:
        - The connection type, e.g. TCPIP.
        - The address of the device, an IP address, hostname, or
          string in the format ``model-serial``.

    Args:
        input_str: The string to check for a VISA resource expression.

    Returns:
        A tuple with the connection information parts.
    """
    retval: Optional[Tuple[str, str]] = None
    if input_str.upper().startswith("ASRL"):
        retval = ("SERIAL", input_str[4:].split("::", 1)[0])
    elif (match := VISA_RESOURCE_EXPRESSION_REGEX.search(input_str.upper())) is not None:
        match_groups_list = list(filter(None, match.groups()))
        for unneeded_part in ("INST", "INST0"):
            if unneeded_part in match_groups_list:
                match_groups_list.remove(unneeded_part)
        # Check if the model is in the USB model lookup
        filtered_usb_model_keys = [
            key
            for key, value in USB_MODEL_ID_LOOKUP.items()
            if value.model_id == match_groups_list[1].lower()
        ]
        if filtered_usb_model_keys:
            # SMU and PSU need to be removed from the string to prevent issues
            match_groups_list[1] = filtered_usb_model_keys[0].replace("SMU", "").replace("PSU", "")
        retval = (match_groups_list[0].rstrip("0"), "-".join(match_groups_list[1:]).lstrip("0X"))
    return retval


# pylint: disable=too-many-branches
def get_model_series(model: str) -> str:  # noqa: PLR0912,C901,PLR0915
    """Get the series string from the full model number.

    Args:
        model: The full model string (ex. MSO58, LPD64).

    Returns:
        The model series string (ex. MSO5, LPD6).
    """
    valid_model_endings = {"A", "B", "C", "D", "LP"}
    model = model.strip().upper()
    model_parts = model.split("-")
    if len(model_parts) == 2:  # noqa: PLR2004
        model = model_parts[0]
    elif len(model_parts) == 3:  # noqa: PLR2004
        model = "MODEL" + model_parts[0]

    # find the model postscript if it exists and format it correctly
    model_postscript = ""
    if len(model_parts) > 1:
        for count, part in enumerate(model_parts):
            # avoid first model part as a postscript will likely never be in the beginning
            if count and len(part) > 1 and all(x.isalpha() for x in part):
                model_postscript = part.capitalize()

    if model.replace("MODEL", "").strip() in {x.upper() for x in SupportedModels.list_values()}:
        # These are special models that don't have differences between the
        # full model number and the model series.
        model_beginning = SupportedModels[model.replace("MODEL", "").strip()].value
        model_end = ""
    else:
        # Get any characters at the end of the model, e.g. 'B', 'LP', 'C'
        model_beginning = ""
        model_end = ""
        if re.search("[0-9]", model):  # if the model contains numbers
            model_end = re.split(r"\d+", model)[-1]  # split on the occurence of each number
        if len(model_end) == 1 and model_end not in valid_model_endings:
            model_end = ""
        model_beginning = ""
        if model_numbers := re.findall(r"\d+", model):
            model_number = int(model_numbers[0])
            if model.startswith("MODEL") or all(x.isdigit() for x in model.rstrip(model_end)):
                model_beginning_idx = model.find(model_numbers[0])
                model_beginning = model[model_beginning_idx:]
                if model_beginning[-1].isalpha():
                    model_beginning = model_beginning[:-1]
                # Convert the model into more specific driver names
                with contextlib.suppress(KeyError):
                    model_beginning = (
                        _KEITHLEY_2_CHAR_MODEL_LOOKUP[model_beginning[:2]] + model_beginning
                    )

                # This block of code can be enabled in the future if specific models need
                # to be combined, e.g. SMU2601B -> SMU2600B
                # Check if this is a specific, unique model series or if it needs to be a
                # more generic driver.
                # RELIC # if not model_postscript and (
                # RELIC #     any(
                # RELIC #         model_beginning.startswith(x)
                # RELIC #         for x in _MODEL_BEGINNINGS_TO_COMBINE
                # RELIC #     )
                # RELIC #     and model_beginning not in _SPECIFIC_DRIVER_SET
                # RELIC # ):
                # RELIC #     # Models without a postscript or which are not part of a
                # RELIC #     # specific driver need to use a more generic driver.
                # RELIC #     model_beginning = model_beginning[:-2] + "00"
            elif model.startswith("AWG52"):
                model_beginning += model[:-1] + "0"
            elif model_number >= 10_000:  # noqa: PLR2004
                if model.startswith("AFG"):
                    model_beginning += model[:5] + "K"
                else:
                    model_beginning += model[:4] + "0K"
            elif 1_000 <= model_number < 10_000:  # noqa: PLR2004
                model_beginning += model[:4] + "K"
            else:
                model_beginning += model[:4]
        elif len(model_parts) >= 2:  # noqa: PLR2004
            model_beginning = model.capitalize()
            model_beginning += model_parts[1]
        else:
            model_beginning = model_parts[0]

    model_series = model_beginning + model_end + model_postscript

    if model_series not in SupportedModels.list_values():
        warnings.warn(
            f'The "{model_series}" model series is not supported by {PACKAGE_NAME}', stacklevel=2
        )

    return model_series


def get_timestamp_string() -> str:
    """Return a string containing the current timestamp."""
    return str(datetime.datetime.now())[:-3]


def get_version(version_string: str) -> Version:
    """Get a Version object from a string.

    Args:
        version_string: The string containing the version to create.

    Returns:
        The Version object.
    """
    version_parts = version_string.split(".")
    try:
        version = Version(version_string)
    except InvalidVersion:
        if all(x.isalpha() for x in version_parts[-1]) and all(
            y.isdigit() for y in version_parts[:-1]
        ):
            version = Version(
                version_string.replace("." + version_parts[-1], "+" + version_parts[-1])
            )
        else:
            raise
    return version


def get_visa_backend(visa_lib_path: str) -> str:
    """Create a human-readable VISA backend name based on the currently used VISA library path.

    Args:
        visa_lib_path: The path to the currently used VISA library.

    Returns:
        A string indicating the current VISA backend.
    """
    visa_name = ""

    system_visa_info = _VISA_SYSTEM_DETAILS
    # noinspection PyTypeChecker
    visa_backends: Dict[str, Any] = system_visa_info["backends"]

    # Figure out which visa is being used
    try:
        if visa_lib_path == "py":
            visa_name = "PyVISA-py"
        else:
            raise KeyError
    except KeyError:
        found_visa = False
        for visa_type in visa_backends:
            for visa_implementation in visa_backends[visa_type]:
                if visa_lib_path in visa_implementation:
                    vendor = visa_backends[visa_type][visa_implementation]["Vendor"]
                    visa_name = "NI-VISA" if "National Instruments" in vendor else f"{vendor} VISA"
                    found_visa = True
                elif visa_lib_path.endswith("yaml") and visa_type == "sim":
                    visa_name = "PyVISA-sim"
                    found_visa = True
            if found_visa:
                break

    return visa_name


def ping_address(address: str, timeout: int = 2) -> str:
    """Ping the given address.

    Args:
        address: The address to ping, either an IP or hostname.
        timeout: The timeout to use, in seconds.

    Returns:
         The response of the ping command.
    """
    ping_command = _create_ping_command(address, timeout)

    try:
        output = subprocess.check_output(shlex.split(ping_command)).decode("utf-8")  # noqa: S603
    except subprocess.CalledProcessError:
        output = ""
    return output.replace("\r\n", "\n")


def print_with_timestamp(message: str, end: str = "\n") -> str:
    """Print and return a string prepended with a timestamp.

    Args:
        message: The message to print.
        end: The end of the line to print.
    """
    message = f"{get_timestamp_string()} - {message}"
    print(message, end=end)
    return message


def sanitize_enum(value: str, enum_class: Type[CustomStrEnum]) -> CustomStrEnum:
    """Sanitize a string value into its enum value.

    Args:
        value: The value to check for.
        enum_class: The Enum class to look in.

    Returns:
        The enum value corresponding to the string value passed in.

    Raises:
        TypeError: Indicates that the value doesn't exist in the enum
    """
    if string_in_enum(value, enum_class):
        sanitized_enum = enum_class[value]
    else:
        msg = (
            f"{value} is not a valid enum of {enum_class.__name__}. "
            f"Valid options are: {enum_class.list_values()}"
        )
        raise TypeError(msg)

    return sanitized_enum


def string_in_enum(value: str, enum_class: EnumMeta) -> bool:
    """Check if a string value exists in an Enum class object.

    Args:
        value: The value to check for.
        enum_class: The Enum class to look in.

    Returns:
        A boolean indicating if the string exists in the Enum.
    """
    try:
        _ = enum_class[value]
    except KeyError:
        return False
    return True


####################################################################################################
# Private Functions
####################################################################################################
def _create_ping_command(address: str, timeout: int = 2) -> str:
    """Create a ping command to ping the given address.

    Args:
        address: The address to ping, either an IP or hostname.
        timeout: The timeout to use, in seconds.

    Returns:
         The response of the ping command.
    """
    validate_address(address)
    # Create the ping command
    if (platform_system := platform.system().lower()) == "windows":
        count_arg = "-n"
        timeout_arg = "-w"
        timeout_val = timeout * 1000  # Windows uses milliseconds
    else:
        count_arg = "-c"
        timeout_arg = "-W" if platform_system == "darwin" else "-w"
        timeout_val = timeout  # Linux/MacOS uses seconds
    return f"ping {address} {count_arg} 1 {timeout_arg} {timeout_val}"


def _configure_visa_object(
    visa_object: MessageBasedResource,
    device_config_entry: DeviceConfigEntry,
    visa_library: str,
) -> MessageBasedResource:
    """Configure settings on the VISA resource object.

    Args:
        visa_object: The VISA resource object to configure.
        device_config_entry: The device config entry.
        visa_library: A string containing the VISA library to use to create a ResourceManager.

    Returns:
        A VISA resource that can be passed into the device driver.
    """
    visa_object.encoding = "utf-8"

    if visa_library.endswith("@sim"):  # pragma: no cover
        visa_object.read_termination = "\n"
        visa_object.write_termination = "\n"

    if isinstance(visa_object, visa.resources.TCPIPSocket):
        # Socket connections seem to need the termination characters specified.
        visa_object.write_termination = "\n"
        visa_object.read_termination = "\n"

        if visa_library != PYVISA_PY_BACKEND:  # pragma: no cover
            visa_object.send_end = True
    elif (
        isinstance(visa_object, visa.resources.SerialInstrument)
        and device_config_entry.serial_config
    ):
        serial_config = device_config_entry.serial_config
        # update serial fields if they were specified, otherwise use the serial visa object defaults
        for name in serial_config.to_dict():
            if (value := getattr(serial_config, name)) is not None:
                setattr(visa_object, name, value)
            else:
                setattr(serial_config, name, getattr(visa_object, name))

    return visa_object



"""A collection of device command classes.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
from ._afg3k_commands import AFG3KCommandConstants, AFG3KCommands, AFG3KMixin
from ._afg3kb_commands import AFG3KBCommandConstants, AFG3KBCommands, AFG3KBMixin
from ._afg3kc_commands import AFG3KCCommandConstants, AFG3KCCommands, AFG3KCMixin
from ._awg5k_commands import AWG5KCommandConstants, AWG5KCommands, AWG5KMixin
from ._awg5kc_commands import AWG5KCCommandConstants, AWG5KCCommands, AWG5KCMixin
from ._awg7k_commands import AWG7KCommandConstants, AWG7KCommands, AWG7KMixin
from ._awg7kc_commands import AWG7KCCommandConstants, AWG7KCCommands, AWG7KCMixin
from ._awg70ka_commands import AWG70KACommandConstants, AWG70KACommands, AWG70KAMixin
from ._awg70kb_commands import AWG70KBCommandConstants, AWG70KBCommands, AWG70KBMixin
from ._awg5200_commands import AWG5200CommandConstants, AWG5200Commands, AWG5200Mixin
from ._daq6510_commands import DAQ6510CommandConstants, DAQ6510Commands, DAQ6510Mixin
from ._dmm6500_commands import DMM6500CommandConstants, DMM6500Commands, DMM6500Mixin
from ._dmm7510_commands import DMM7510CommandConstants, DMM7510Commands, DMM7510Mixin
from ._dpo2k_commands import DPO2KCommandConstants, DPO2KCommands, DPO2KMixin
from ._dpo2kb_commands import DPO2KBCommandConstants, DPO2KBCommands, DPO2KBMixin
from ._dpo4k_commands import DPO4KCommandConstants, DPO4KCommands, DPO4KMixin
from ._dpo4kb_commands import DPO4KBCommandConstants, DPO4KBCommands, DPO4KBMixin
from ._dpo5kb_commands import DPO5KBCommandConstants, DPO5KBCommands, DPO5KBMixin
from ._dpo7kc_commands import DPO7KCCommandConstants, DPO7KCCommands, DPO7KCMixin
from ._dpo70kc_commands import DPO70KCCommandConstants, DPO70KCCommands, DPO70KCMixin
from ._dpo70kd_commands import DPO70KDCommandConstants, DPO70KDCommands, DPO70KDMixin
from ._dpo70kdx_commands import DPO70KDXCommandConstants, DPO70KDXCommands, DPO70KDXMixin
from ._dpo70ksx_commands import DPO70KSXCommandConstants, DPO70KSXCommands, DPO70KSXMixin
from ._dsa70kc_commands import DSA70KCCommandConstants, DSA70KCCommands, DSA70KCMixin
from ._dsa70kd_commands import DSA70KDCommandConstants, DSA70KDCommands, DSA70KDMixin
from ._helpers.generic_commands import NoDeviceProvidedError
from ._lpd6_commands import LPD6CommandConstants, LPD6Commands, LPD6Mixin
from ._mdo3_commands import MDO3CommandConstants, MDO3Commands, MDO3Mixin
from ._mdo3k_commands import MDO3KCommandConstants, MDO3KCommands, MDO3KMixin
from ._mdo4k_commands import MDO4KCommandConstants, MDO4KCommands, MDO4KMixin
from ._mdo4kb_commands import MDO4KBCommandConstants, MDO4KBCommands, MDO4KBMixin
from ._mdo4kc_commands import MDO4KCCommandConstants, MDO4KCCommands, MDO4KCMixin
from ._mso2_commands import MSO2CommandConstants, MSO2Commands, MSO2Mixin
from ._mso2k_commands import MSO2KCommandConstants, MSO2KCommands, MSO2KMixin
from ._mso2kb_commands import MSO2KBCommandConstants, MSO2KBCommands, MSO2KBMixin
from ._mso4_commands import MSO4CommandConstants, MSO4Commands, MSO4Mixin
from ._mso4k_commands import MSO4KCommandConstants, MSO4KCommands, MSO4KMixin
from ._mso4kb_commands import MSO4KBCommandConstants, MSO4KBCommands, MSO4KBMixin
from ._mso5_commands import MSO5CommandConstants, MSO5Commands, MSO5Mixin
from ._mso5b_commands import MSO5BCommandConstants, MSO5BCommands, MSO5BMixin
from ._mso5kb_commands import MSO5KBCommandConstants, MSO5KBCommands, MSO5KBMixin
from ._mso5lp_commands import MSO5LPCommandConstants, MSO5LPCommands, MSO5LPMixin
from ._mso6_commands import MSO6CommandConstants, MSO6Commands, MSO6Mixin
from ._mso6b_commands import MSO6BCommandConstants, MSO6BCommands, MSO6BMixin
from ._mso70kc_commands import MSO70KCCommandConstants, MSO70KCCommands, MSO70KCMixin
from ._mso70kdx_commands import MSO70KDXCommandConstants, MSO70KDXCommands, MSO70KDXMixin
from ._smu2450_commands import SMU2450CommandConstants, SMU2450Commands, SMU2450Mixin
from ._smu2460_commands import SMU2460CommandConstants, SMU2460Commands, SMU2460Mixin
from ._smu2461_commands import SMU2461CommandConstants, SMU2461Commands, SMU2461Mixin
from ._smu2470_commands import SMU2470CommandConstants, SMU2470Commands, SMU2470Mixin
from ._smu2601b_commands import SMU2601BCommandConstants, SMU2601BCommands, SMU2601BMixin
from ._smu2601b_pulse_commands import (
    SMU2601BPulseCommandConstants,
    SMU2601BPulseCommands,
    SMU2601BPulseMixin,
)
from ._smu2602b_commands import SMU2602BCommandConstants, SMU2602BCommands, SMU2602BMixin
from ._smu2604b_commands import SMU2604BCommandConstants, SMU2604BCommands, SMU2604BMixin
from ._smu2606b_commands import SMU2606BCommandConstants, SMU2606BCommands, SMU2606BMixin
from ._smu2611b_commands import SMU2611BCommandConstants, SMU2611BCommands, SMU2611BMixin
from ._smu2612b_commands import SMU2612BCommandConstants, SMU2612BCommands, SMU2612BMixin
from ._smu2614b_commands import SMU2614BCommandConstants, SMU2614BCommands, SMU2614BMixin
from ._smu2634b_commands import SMU2634BCommandConstants, SMU2634BCommands, SMU2634BMixin
from ._smu2635b_commands import SMU2635BCommandConstants, SMU2635BCommands, SMU2635BMixin
from ._smu2636b_commands import SMU2636BCommandConstants, SMU2636BCommands, SMU2636BMixin
from ._smu2651a_commands import SMU2651ACommandConstants, SMU2651ACommands, SMU2651AMixin
from ._smu2657a_commands import SMU2657ACommandConstants, SMU2657ACommands, SMU2657AMixin
from ._ss3706a_commands import SS3706ACommandConstants, SS3706ACommands, SS3706AMixin

__all__ = [
    "AFG3KBCommandConstants",
    "AFG3KBCommands",
    "AFG3KBMixin",
    "AFG3KCCommandConstants",
    "AFG3KCCommands",
    "AFG3KCMixin",
    "AFG3KCommandConstants",
    "AFG3KCommands",
    "AFG3KMixin",
    "AWG5200CommandConstants",
    "AWG5200Commands",
    "AWG5200Mixin",
    "AWG5KCCommandConstants",
    "AWG5KCCommands",
    "AWG5KCMixin",
    "AWG5KCommandConstants",
    "AWG5KCommands",
    "AWG5KMixin",
    "AWG70KACommandConstants",
    "AWG70KACommands",
    "AWG70KAMixin",
    "AWG70KBCommandConstants",
    "AWG70KBCommands",
    "AWG70KBMixin",
    "AWG7KCCommandConstants",
    "AWG7KCCommands",
    "AWG7KCMixin",
    "AWG7KCommandConstants",
    "AWG7KCommands",
    "AWG7KMixin",
    "DAQ6510CommandConstants",
    "DAQ6510Commands",
    "DAQ6510Mixin",
    "DMM6500CommandConstants",
    "DMM6500Commands",
    "DMM6500Mixin",
    "DMM7510CommandConstants",
    "DMM7510Commands",
    "DMM7510Mixin",
    "DPO2KBCommandConstants",
    "DPO2KBCommands",
    "DPO2KBMixin",
    "DPO2KCommandConstants",
    "DPO2KCommands",
    "DPO2KMixin",
    "DPO4KBCommandConstants",
    "DPO4KBCommands",
    "DPO4KBMixin",
    "DPO4KCommandConstants",
    "DPO4KCommands",
    "DPO4KMixin",
    "DPO5KBCommandConstants",
    "DPO5KBCommands",
    "DPO5KBMixin",
    "DPO70KCCommandConstants",
    "DPO70KCCommands",
    "DPO70KCMixin",
    "DPO70KDCommandConstants",
    "DPO70KDCommands",
    "DPO70KDMixin",
    "DPO70KDXCommandConstants",
    "DPO70KDXCommands",
    "DPO70KDXMixin",
    "DPO70KSXCommandConstants",
    "DPO70KSXCommands",
    "DPO70KSXMixin",
    "DPO7KCCommandConstants",
    "DPO7KCCommands",
    "DPO7KCMixin",
    "DSA70KCCommandConstants",
    "DSA70KCCommands",
    "DSA70KCMixin",
    "DSA70KDCommandConstants",
    "DSA70KDCommands",
    "DSA70KDMixin",
    "LPD6CommandConstants",
    "LPD6Commands",
    "LPD6Mixin",
    "MDO3CommandConstants",
    "MDO3Commands",
    "MDO3KCommandConstants",
    "MDO3KCommands",
    "MDO3KMixin",
    "MDO3Mixin",
    "MDO4KBCommandConstants",
    "MDO4KBCommands",
    "MDO4KBMixin",
    "MDO4KCCommandConstants",
    "MDO4KCCommands",
    "MDO4KCMixin",
    "MDO4KCommandConstants",
    "MDO4KCommands",
    "MDO4KMixin",
    "MSO2CommandConstants",
    "MSO2Commands",
    "MSO2KBCommandConstants",
    "MSO2KBCommands",
    "MSO2KBMixin",
    "MSO2KCommandConstants",
    "MSO2KCommands",
    "MSO2KMixin",
    "MSO2Mixin",
    "MSO4CommandConstants",
    "MSO4Commands",
    "MSO4KBCommandConstants",
    "MSO4KBCommands",
    "MSO4KBMixin",
    "MSO4KCommandConstants",
    "MSO4KCommands",
    "MSO4KMixin",
    "MSO4Mixin",
    "MSO5BCommandConstants",
    "MSO5BCommands",
    "MSO5BMixin",
    "MSO5CommandConstants",
    "MSO5Commands",
    "MSO5KBCommandConstants",
    "MSO5KBCommands",
    "MSO5KBMixin",
    "MSO5LPCommandConstants",
    "MSO5LPCommands",
    "MSO5LPMixin",
    "MSO5Mixin",
    "MSO6BCommandConstants",
    "MSO6BCommands",
    "MSO6BMixin",
    "MSO6CommandConstants",
    "MSO6Commands",
    "MSO6Mixin",
    "MSO70KCCommandConstants",
    "MSO70KCCommands",
    "MSO70KCMixin",
    "MSO70KDXCommandConstants",
    "MSO70KDXCommands",
    "MSO70KDXMixin",
    "NoDeviceProvidedError",
    "SMU2450CommandConstants",
    "SMU2450Commands",
    "SMU2450Mixin",
    "SMU2460CommandConstants",
    "SMU2460Commands",
    "SMU2460Mixin",
    "SMU2461CommandConstants",
    "SMU2461Commands",
    "SMU2461Mixin",
    "SMU2470CommandConstants",
    "SMU2470Commands",
    "SMU2470Mixin",
    "SMU2601BCommandConstants",
    "SMU2601BCommands",
    "SMU2601BMixin",
    "SMU2601BPulseCommandConstants",
    "SMU2601BPulseCommands",
    "SMU2601BPulseMixin",
    "SMU2602BCommandConstants",
    "SMU2602BCommands",
    "SMU2602BMixin",
    "SMU2604BCommandConstants",
    "SMU2604BCommands",
    "SMU2604BMixin",
    "SMU2606BCommandConstants",
    "SMU2606BCommands",
    "SMU2606BMixin",
    "SMU2611BCommandConstants",
    "SMU2611BCommands",
    "SMU2611BMixin",
    "SMU2612BCommandConstants",
    "SMU2612BCommands",
    "SMU2612BMixin",
    "SMU2614BCommandConstants",
    "SMU2614BCommands",
    "SMU2614BMixin",
    "SMU2634BCommandConstants",
    "SMU2634BCommands",
    "SMU2634BMixin",
    "SMU2635BCommandConstants",
    "SMU2635BCommands",
    "SMU2635BMixin",
    "SMU2636BCommandConstants",
    "SMU2636BCommands",
    "SMU2636BMixin",
    "SMU2651ACommandConstants",
    "SMU2651ACommands",
    "SMU2651AMixin",
    "SMU2657ACommandConstants",
    "SMU2657ACommands",
    "SMU2657AMixin",
    "SS3706ACommandConstants",
    "SS3706ACommands",
    "SS3706AMixin",
]



"""The setup_1 commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SETUP<x>:DATE?
    - SETUP<x>:LABEL <Qstring>
    - SETUP<x>:TIME?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SetupItemTime(SCPICmdRead):
    """The ``SETUP<x>:TIME`` command.

    **Description:**
        - Returns the time when the oscilloscope setup was saved for the specified channel <x>,
          where x can be 1 through 10.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP<x>:TIME?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP<x>:TIME?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SETUP<x>:TIME?
    """


class SetupItemLabel(SCPICmdWrite):
    """The ``SETUP<x>:LABEL`` command.

    **Description:**
        - This command specifies the setup label for the specified channel <x>, where x can be 1
          through 10.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SETUP<x>:LABEL value`` command.

    **SCPI Syntax:**

    ::

        - SETUP<x>:LABEL <Qstring>

    **Info:**
        - ``<Qstring>`` is an alphanumeric string of characters, enclosed in quotes, that defines
          the label text for SETUP<x>. The length of the string is limited to 30 characters.
    """


class SetupItemDate(SCPICmdRead):
    """The ``SETUP<x>:DATE`` command.

    **Description:**
        - Returns the date when the oscilloscope setup was saved for the specified channel <x>,
          where x can be 1 through 10.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP<x>:DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP<x>:DATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SETUP<x>:DATE?
    """


class SetupItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SETUP<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.date``: The ``SETUP<x>:DATE`` command.
        - ``.label``: The ``SETUP<x>:LABEL`` command.
        - ``.time``: The ``SETUP<x>:TIME`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SETUP<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._date = SetupItemDate(device, f"{self._cmd_syntax}:DATE")
        self._label = SetupItemLabel(device, f"{self._cmd_syntax}:LABEL")
        self._time = SetupItemTime(device, f"{self._cmd_syntax}:TIME")

    @property
    def date(self) -> SetupItemDate:
        """Return the ``SETUP<x>:DATE`` command.

        **Description:**
            - Returns the date when the oscilloscope setup was saved for the specified channel <x>,
              where x can be 1 through 10.

        **Usage:**
            - Using the ``.query()`` method will send the ``SETUP<x>:DATE?`` query.
            - Using the ``.verify(value)`` method will send the ``SETUP<x>:DATE?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SETUP<x>:DATE?
        """
        return self._date

    @property
    def label(self) -> SetupItemLabel:
        """Return the ``SETUP<x>:LABEL`` command.

        **Description:**
            - This command specifies the setup label for the specified channel <x>, where x can be 1
              through 10.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SETUP<x>:LABEL value`` command.

        **SCPI Syntax:**

        ::

            - SETUP<x>:LABEL <Qstring>

        **Info:**
            - ``<Qstring>`` is an alphanumeric string of characters, enclosed in quotes, that
              defines the label text for SETUP<x>. The length of the string is limited to 30
              characters.
        """
        return self._label

    @property
    def time(self) -> SetupItemTime:
        """Return the ``SETUP<x>:TIME`` command.

        **Description:**
            - Returns the time when the oscilloscope setup was saved for the specified channel <x>,
              where x can be 1 through 10.

        **Usage:**
            - Using the ``.query()`` method will send the ``SETUP<x>:TIME?`` query.
            - Using the ``.verify(value)`` method will send the ``SETUP<x>:TIME?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SETUP<x>:TIME?
        """
        return self._time



"""Digital Multimeter package init file."""
from tm_devices.drivers.pi.digital_multimeters.digital_multimeter import DigitalMultimeter
from tm_devices.drivers.pi.digital_multimeters.dmm6500.dmm6500 import DMM6500
from tm_devices.drivers.pi.digital_multimeters.dmm7500.dmm7500 import DMM7500

__all__ = [
    "DigitalMultimeter",
    "DMM6500",
    "DMM7500",
]



"""The data commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - DATa {INIT|SNAp}
    - DATa:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
    - DATa:COMPosition:AVAILable?
    - DATa:COMPosition?
    - DATa:DESTination REF<x>
    - DATa:DESTination?
    - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary}
    - DATa:ENCdg?
    - DATa:RESOlution {FULL|REDUced}
    - DATa:RESOlution?
    - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>}
    - DATa:SOUrce?
    - DATa:STARt <NR1>
    - DATa:STARt?
    - DATa:STOP <NR1>
    - DATa:STOP?
    - DATa:WIDth <NR1>
    - DATa:WIDth?
    - DATa?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class DataWidth(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:WIDth`` command.

    **Description:**
        - This command specifies the width, in bytes per point, for waveform data transferred from
          the instrument via the CURVe? query. (This command is synonymous with
          ``WFMOutpre:BYT_Nr``.)

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:WIDth?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:WIDth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:WIDth value`` command.

    **SCPI Syntax:**

    ::

        - DATa:WIDth <NR1>
        - DATa:WIDth?

    **Info:**
        - ``<NR1>`` is an integer that indicates the number of bytes per point for the outgoing
          waveform data when queried using the CURVe? command.
    """


class DataStop(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:STOP`` command.

    **Description:**
        - This command sets or queries the last data point that will be transferred when using the
          CURVE? query. When using the CURVE command, ``DATa:STOP`` is ignored. This command allows
          for the transfer of partial waveforms to the controller. If <NR1> is greater than the
          record length, then data will be transferred up to the record length. If both
          ``DATa:STARt`` and ``DATa:STOP`` are greater than the record length, the last data point
          in the record is returned. ``DATa:STARt`` and ``DATa:STOP`` are order independent. When
          ``DATa:STOP`` is less than ``DATa:STARt``, the values will be swapped internally for the
          CURVE? query. If you always want to transfer complete waveforms, set ``DATa:STARt`` to 1
          and ``DATa:STOP`` to the maximum record length, or larger. Changes to the record length
          value are not automatically reflected in the ``DATa:STOP`` value. As record length is
          varied, the ``DATa:STOP`` value must be explicitly changed to ensure the entire record is
          transmitted. In other words, curve results will not automatically and correctly reflect
          increases in record length if the distance from ``DATa:STARt`` to ``DATa:STOP`` stays
          smaller than the increased record length.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:STOP?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:STOP?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:STOP value`` command.

    **SCPI Syntax:**

    ::

        - DATa:STOP <NR1>
        - DATa:STOP?

    **Info:**
        - ``<NR1>`` is the last data point that will be transferred, which ranges from 1 to the
          record length.
    """


class DataStart(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:STARt`` command.

    **Description:**
        - This command sets or queries the starting data point for waveform transfer. This command
          allows for the transfer of partial waveforms to and from the instrument. Data will be
          transferred from <NR1> to ``DATa:STOP`` or the record length, whichever is less. If <NR1>
          is greater than the record length, the last data point in the record is transferred.
          ``DATa:STARt`` and ``DATa:STOP`` are order independent. When ``DATa:STOP`` is greater than
          ``DATa:STARt``, the values will be swapped internally for the CURVE? query.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:STARt?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:STARt value`` command.

    **SCPI Syntax:**

    ::

        - DATa:STARt <NR1>
        - DATa:STARt?

    **Info:**
        - ``<NR1>`` is the first data point that will be transferred, which ranges from 1 to the
          record length.
    """


class DataSource(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:SOUrce`` command.

    **Description:**
        - Sets or returns the location of the waveform data transferred from the oscilloscope by the
          CURVE? query.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>}
        - DATa:SOUrce?

    **Info:**
        - ``CH<x>`` specifies which analog channel data will be transferred from the oscilloscope to
          the controller, channels 1 through 4. x has a minimum of 1 and a maximum of 4.
        - ``MATH`` specifies that the Math waveform data will be transferred from the oscilloscope
          to the controller.
        - ``REF<x>`` specifies which Reference waveform data will be transferred from the
          oscilloscope to the controller, waveforms, 1 through 4. x has a minimum of 1 and a maximum
          of 2.
        - ``D<x>`` specifies that the returned waveform data is packaged as 4 bytes per sample,
          representing the states of all digital channels (D0-D15) and the logic states of the
          analog channels (Ch1-Ch4). When the ``:DATA:SOURCE`` is set to DIGITAL, the
          ``:DATA:WIDTH`` is automatically set to 4. x has a minimum of 0 and a maximum of 15.
    """


class DataResolution(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:RESOlution`` command.

    **Description:**
        - Sets or returns whether the CURVE? query returns full resolution records (acquired data)
          or reduced resolution records (filtered/displayed data).

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:RESOlution?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:RESOlution?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:RESOlution value`` command.

    **SCPI Syntax:**

    ::

        - DATa:RESOlution {FULL|REDUced}
        - DATa:RESOlution?

    **Info:**
        - ``FULL`` sets the instrument to return the full undecimated record acquired by the
          instrument. The full resolution records are not subject to the effects of FilterVu. Full
          resolution record lengths are 100,000, 125,000, 1,000,000 or 1,250,000 points.
        - ``REDUced`` sets the instrument to return short waveforms that are decimated from the full
          acquisition record. Reduced records are subject to the low pass filtering effects of
          FilterVu settings. Reduced record lengths range from 800 to 6250 points, depending on the
          acquisition settings.
    """


class DataEncdg(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:ENCdg`` command.

    **Description:**
        - Sets or returns the format of outgoing waveform data. This command is equivalent to
          setting ``WFMOUTPRE:ENCDG``, ``WFMOUTPRE:BN_FMT``, ``andWFMINPRE:FILTERFREQ``. Setting the
          ``DATa:ENGdg`` value causes the corresponding WFMOutpre values to be updated and
          conversely.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:ENCdg?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:ENCdg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:ENCdg value`` command.

    **SCPI Syntax:**

    ::

        - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary}
        - DATa:ENCdg?

    **Info:**
        - ``ASCIi`` specifies the ASCII representation for waveform data points. If ASCII is the
          value, then ``:BN_Fmt`` and ``:BYT_Or`` are ignored.
        - ``FAStest`` specifies that the data be sent in the fastest possible manner consistent with
          maintaining accuracy and is interpreted with respect to the waveform specified by
          ``DATA:SOURCE``.
        - ``RIBinary`` specifies signed integer data point representation with the most significant
          byte transferred first.
        - ``RPBinary`` specifies the positive integer data-point representation, with the most
          significant byte transferred first.
        - ``SRIbinary`` is the same as RIBinary except that the byte order is swapped, meaning that
          the least significant byte is transferred first. This format is useful when transferring
          data to IBM compatible PCs.
        - ``SRPbinary`` is the same as RPBinary except that the byte order is swapped, meaning that
          the least significant byte is transferred first. This format is useful when transferring
          data to PCs.
    """


class DataDestination(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:DESTination`` command.

    **Description:**
        - This command specifies the reference memory location (REF1-4) for storing waveform data
          transferred into the oscilloscope using the CURVE command.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:DESTination?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:DESTination?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:DESTination value`` command.

    **SCPI Syntax:**

    ::

        - DATa:DESTination REF<x>
        - DATa:DESTination?

    **Info:**
        - ``REF<x>`` is the reference location where the waveform will be stored.
    """


class DataCompositionAvailable(SCPICmdRead):
    """The ``DATa:COMPosition:AVAILable`` command.

    **Description:**
        - Lists the waveform data types that are available for return from the instrument under the
          current instrument settings.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:COMPosition:AVAILable?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:COMPosition:AVAILable?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - DATa:COMPosition:AVAILable?
    """


class DataComposition(SCPICmdWrite, SCPICmdRead):
    """The ``DATa:COMPosition`` command.

    **Description:**
        - Sets or returns the type of data that the CURVE? query returns.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa:COMPosition?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa:COMPosition?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa:COMPosition value`` command.

    **SCPI Syntax:**

    ::

        - DATa:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
        - DATa:COMPosition?

    **Info:**
        - ``COMPOSITE_YT`` is of native width 8-bits and is written as 8-bit data in ISF files and
          REF waveforms. The data can be queried as 1- or 2-byte data in CURVE? queries by setting
          the data width to 1 or 2.
        - ``COMPOSITE_ENV`` is min/max pair data. Its native width is 8-bits and is written as 8-bit
          data in ISF files and REF waveforms. The data can be queried as 1- or 2- byte data in
          CURVE? queries by setting the data width to 1 or 2.
        - ``SINGULAR_YT`` is 16-bit data which originates as averaged acquired data or math data. It
          is written to ISF files and REF waveforms as 16-bit data. The data can be queried as 1- or
          2- byte data in CURVE? queries by setting the data width to 1 or 2.

    Properties:
        - ``.available``: The ``DATa:COMPosition:AVAILable`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._available = DataCompositionAvailable(device, f"{self._cmd_syntax}:AVAILable")

    @property
    def available(self) -> DataCompositionAvailable:
        """Return the ``DATa:COMPosition:AVAILable`` command.

        **Description:**
            - Lists the waveform data types that are available for return from the instrument under
              the current instrument settings.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:COMPosition:AVAILable?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:COMPosition:AVAILable?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - DATa:COMPosition:AVAILable?
        """
        return self._available


#  pylint: disable=too-many-instance-attributes
class Data(SCPICmdWrite, SCPICmdRead):
    """The ``DATa`` command.

    **Description:**
        - This command sets or queries the format and location of the waveform data that is
          transferred with the CURVE command.

    **Usage:**
        - Using the ``.query()`` method will send the ``DATa?`` query.
        - Using the ``.verify(value)`` method will send the ``DATa?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DATa value`` command.

    **SCPI Syntax:**

    ::

        - DATa {INIT|SNAp}
        - DATa?

    **Info:**
        - ``INIT`` initializes the waveform data parameters to their factory defaults except for
          ``DATa:STOP``, which isset to the current acquisition record length.
        - ``SNAp`` Sets ``DATa:STARt`` and ``DATa:STOP`` to match the current waveform cursor
          positions of WAVEVIEW1 CURSOR1 if these waveform cursors are currently on. If these
          waveform cursors are not on when the ``DATa SNAp`` command is sent, it is silently ignored
          and ``DATa:STARt`` and ``:STOP`` remain unchanged.

    Properties:
        - ``.composition``: The ``DATa:COMPosition`` command.
        - ``.destination``: The ``DATa:DESTination`` command.
        - ``.encdg``: The ``DATa:ENCdg`` command.
        - ``.resolution``: The ``DATa:RESOlution`` command.
        - ``.source``: The ``DATa:SOUrce`` command.
        - ``.start``: The ``DATa:STARt`` command.
        - ``.stop``: The ``DATa:STOP`` command.
        - ``.width``: The ``DATa:WIDth`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "DATa") -> None:
        super().__init__(device, cmd_syntax)
        self._composition = DataComposition(device, f"{self._cmd_syntax}:COMPosition")
        self._destination = DataDestination(device, f"{self._cmd_syntax}:DESTination")
        self._encdg = DataEncdg(device, f"{self._cmd_syntax}:ENCdg")
        self._resolution = DataResolution(device, f"{self._cmd_syntax}:RESOlution")
        self._source = DataSource(device, f"{self._cmd_syntax}:SOUrce")
        self._start = DataStart(device, f"{self._cmd_syntax}:STARt")
        self._stop = DataStop(device, f"{self._cmd_syntax}:STOP")
        self._width = DataWidth(device, f"{self._cmd_syntax}:WIDth")

    @property
    def composition(self) -> DataComposition:
        """Return the ``DATa:COMPosition`` command.

        **Description:**
            - Sets or returns the type of data that the CURVE? query returns.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:COMPosition?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:COMPosition?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:COMPosition value`` command.

        **SCPI Syntax:**

        ::

            - DATa:COMPosition {COMPOSITE_YT|COMPOSITE_ENV|SINGULAR_YT}
            - DATa:COMPosition?

        **Info:**
            - ``COMPOSITE_YT`` is of native width 8-bits and is written as 8-bit data in ISF files
              and REF waveforms. The data can be queried as 1- or 2-byte data in CURVE? queries by
              setting the data width to 1 or 2.
            - ``COMPOSITE_ENV`` is min/max pair data. Its native width is 8-bits and is written as
              8-bit data in ISF files and REF waveforms. The data can be queried as 1- or 2- byte
              data in CURVE? queries by setting the data width to 1 or 2.
            - ``SINGULAR_YT`` is 16-bit data which originates as averaged acquired data or math
              data. It is written to ISF files and REF waveforms as 16-bit data. The data can be
              queried as 1- or 2- byte data in CURVE? queries by setting the data width to 1 or 2.

        Sub-properties:
            - ``.available``: The ``DATa:COMPosition:AVAILable`` command.
        """
        return self._composition

    @property
    def destination(self) -> DataDestination:
        """Return the ``DATa:DESTination`` command.

        **Description:**
            - This command specifies the reference memory location (REF1-4) for storing waveform
              data transferred into the oscilloscope using the CURVE command.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:DESTination?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:DESTination?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:DESTination value`` command.

        **SCPI Syntax:**

        ::

            - DATa:DESTination REF<x>
            - DATa:DESTination?

        **Info:**
            - ``REF<x>`` is the reference location where the waveform will be stored.
        """
        return self._destination

    @property
    def encdg(self) -> DataEncdg:
        """Return the ``DATa:ENCdg`` command.

        **Description:**
            - Sets or returns the format of outgoing waveform data. This command is equivalent to
              setting ``WFMOUTPRE:ENCDG``, ``WFMOUTPRE:BN_FMT``, ``andWFMINPRE:FILTERFREQ``. Setting
              the ``DATa:ENGdg`` value causes the corresponding WFMOutpre values to be updated and
              conversely.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:ENCdg?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:ENCdg?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:ENCdg value`` command.

        **SCPI Syntax:**

        ::

            - DATa:ENCdg {ASCIi|FAStest|RIBinary|RPBinary|SRIbinary|SRPbinary}
            - DATa:ENCdg?

        **Info:**
            - ``ASCIi`` specifies the ASCII representation for waveform data points. If ASCII is the
              value, then ``:BN_Fmt`` and ``:BYT_Or`` are ignored.
            - ``FAStest`` specifies that the data be sent in the fastest possible manner consistent
              with maintaining accuracy and is interpreted with respect to the waveform specified by
              ``DATA:SOURCE``.
            - ``RIBinary`` specifies signed integer data point representation with the most
              significant byte transferred first.
            - ``RPBinary`` specifies the positive integer data-point representation, with the most
              significant byte transferred first.
            - ``SRIbinary`` is the same as RIBinary except that the byte order is swapped, meaning
              that the least significant byte is transferred first. This format is useful when
              transferring data to IBM compatible PCs.
            - ``SRPbinary`` is the same as RPBinary except that the byte order is swapped, meaning
              that the least significant byte is transferred first. This format is useful when
              transferring data to PCs.
        """
        return self._encdg

    @property
    def resolution(self) -> DataResolution:
        """Return the ``DATa:RESOlution`` command.

        **Description:**
            - Sets or returns whether the CURVE? query returns full resolution records (acquired
              data) or reduced resolution records (filtered/displayed data).

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:RESOlution?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:RESOlution?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:RESOlution value`` command.

        **SCPI Syntax:**

        ::

            - DATa:RESOlution {FULL|REDUced}
            - DATa:RESOlution?

        **Info:**
            - ``FULL`` sets the instrument to return the full undecimated record acquired by the
              instrument. The full resolution records are not subject to the effects of FilterVu.
              Full resolution record lengths are 100,000, 125,000, 1,000,000 or 1,250,000 points.
            - ``REDUced`` sets the instrument to return short waveforms that are decimated from the
              full acquisition record. Reduced records are subject to the low pass filtering effects
              of FilterVu settings. Reduced record lengths range from 800 to 6250 points, depending
              on the acquisition settings.
        """
        return self._resolution

    @property
    def source(self) -> DataSource:
        """Return the ``DATa:SOUrce`` command.

        **Description:**
            - Sets or returns the location of the waveform data transferred from the oscilloscope by
              the CURVE? query.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:SOUrce?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:SOUrce value`` command.

        **SCPI Syntax:**

        ::

            - DATa:SOUrce {CH<x>|MATH|REF<x>|D<x>}
            - DATa:SOUrce?

        **Info:**
            - ``CH<x>`` specifies which analog channel data will be transferred from the
              oscilloscope to the controller, channels 1 through 4. x has a minimum of 1 and a
              maximum of 4.
            - ``MATH`` specifies that the Math waveform data will be transferred from the
              oscilloscope to the controller.
            - ``REF<x>`` specifies which Reference waveform data will be transferred from the
              oscilloscope to the controller, waveforms, 1 through 4. x has a minimum of 1 and a
              maximum of 2.
            - ``D<x>`` specifies that the returned waveform data is packaged as 4 bytes per sample,
              representing the states of all digital channels (D0-D15) and the logic states of the
              analog channels (Ch1-Ch4). When the ``:DATA:SOURCE`` is set to DIGITAL, the
              ``:DATA:WIDTH`` is automatically set to 4. x has a minimum of 0 and a maximum of 15.
        """
        return self._source

    @property
    def start(self) -> DataStart:
        """Return the ``DATa:STARt`` command.

        **Description:**
            - This command sets or queries the starting data point for waveform transfer. This
              command allows for the transfer of partial waveforms to and from the instrument. Data
              will be transferred from <NR1> to ``DATa:STOP`` or the record length, whichever is
              less. If <NR1> is greater than the record length, the last data point in the record is
              transferred. ``DATa:STARt`` and ``DATa:STOP`` are order independent. When
              ``DATa:STOP`` is greater than ``DATa:STARt``, the values will be swapped internally
              for the CURVE? query.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:STARt?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:STARt value`` command.

        **SCPI Syntax:**

        ::

            - DATa:STARt <NR1>
            - DATa:STARt?

        **Info:**
            - ``<NR1>`` is the first data point that will be transferred, which ranges from 1 to the
              record length.
        """
        return self._start

    @property
    def stop(self) -> DataStop:
        """Return the ``DATa:STOP`` command.

        **Description:**
            - This command sets or queries the last data point that will be transferred when using
              the CURVE? query. When using the CURVE command, ``DATa:STOP`` is ignored. This command
              allows for the transfer of partial waveforms to the controller. If <NR1> is greater
              than the record length, then data will be transferred up to the record length. If both
              ``DATa:STARt`` and ``DATa:STOP`` are greater than the record length, the last data
              point in the record is returned. ``DATa:STARt`` and ``DATa:STOP`` are order
              independent. When ``DATa:STOP`` is less than ``DATa:STARt``, the values will be
              swapped internally for the CURVE? query. If you always want to transfer complete
              waveforms, set ``DATa:STARt`` to 1 and ``DATa:STOP`` to the maximum record length, or
              larger. Changes to the record length value are not automatically reflected in the
              ``DATa:STOP`` value. As record length is varied, the ``DATa:STOP`` value must be
              explicitly changed to ensure the entire record is transmitted. In other words, curve
              results will not automatically and correctly reflect increases in record length if the
              distance from ``DATa:STARt`` to ``DATa:STOP`` stays smaller than the increased record
              length.

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:STOP?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:STOP?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:STOP value`` command.

        **SCPI Syntax:**

        ::

            - DATa:STOP <NR1>
            - DATa:STOP?

        **Info:**
            - ``<NR1>`` is the last data point that will be transferred, which ranges from 1 to the
              record length.
        """
        return self._stop

    @property
    def width(self) -> DataWidth:
        """Return the ``DATa:WIDth`` command.

        **Description:**
            - This command specifies the width, in bytes per point, for waveform data transferred
              from the instrument via the CURVe? query. (This command is synonymous with
              ``WFMOutpre:BYT_Nr``.)

        **Usage:**
            - Using the ``.query()`` method will send the ``DATa:WIDth?`` query.
            - Using the ``.verify(value)`` method will send the ``DATa:WIDth?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DATa:WIDth value`` command.

        **SCPI Syntax:**

        ::

            - DATa:WIDth <NR1>
            - DATa:WIDth?

        **Info:**
            - ``<NR1>`` is an integer that indicates the number of bytes per point for the outgoing
              waveform data when queried using the CURVe? command.
        """
        return self._width



"""2200 Base device driver for the 22xx family of power supplies."""
from functools import cached_property
from typing import Tuple

from packaging.version import Version

from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.pi.power_supplies.power_supply import PowerSupplyUnit
from tm_devices.helpers import get_version


@family_base_class
class PSU2200(PowerSupplyUnit):
    """2200 Base device driver for the 22xx family of power supplies."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def all_channel_names_list(self) -> Tuple[str, ...]:
        """Return a tuple containing all the channel names."""
        return tuple(f"SOURCE{x+1}" for x in range(self.total_channels))

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        return max(1, int(self.model[2])) if self.model[2].isdecimal() else 1

    ################################################################################################
    # Public Methods
    ################################################################################################
    @cached_property
    def fpga_version(self) -> Version:
        """Return the fpga version of the device."""
        id_string_parts = self.idn_string.split(",")
        sw_version = id_string_parts[-1]
        if "-" in sw_version:
            split_sw = sw_version.split("-")
            return get_version(split_sw[1])
        return Version("0")  # pragma: no cover

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Perform the actual rebooting code."""
        # TODO: implement
        raise NotImplementedError(
            f"``.reboot()`` is not yet implemented for the {self.__class__.__name__} driver"
        )



"""The teklink commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - TEKLink:CONNection?
    - TEKLink:REFClk {ON|OFF|PENDING}
    - TEKLink:REFClk?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class TeklinkRefclk(SCPICmdWrite, SCPICmdRead):
    """The ``TEKLink:REFClk`` command.

    **Description:**
        - This command sets or queries the current state of the intrument's TekLink Reference
          Output.

    **Usage:**
        - Using the ``.query()`` method will send the ``TEKLink:REFClk?`` query.
        - Using the ``.verify(value)`` method will send the ``TEKLink:REFClk?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TEKLink:REFClk value`` command.

    **SCPI Syntax:**

    ::

        - TEKLink:REFClk {ON|OFF|PENDING}
        - TEKLink:REFClk?

    **Info:**
        - ``ON`` specifies the on state of the TekLink Reference Output.
        - ``OFF`` specifies the off state of the TekLink Reference Output.
        - ``PENDING`` specifies the pending state of the TekLink Reference Output.
    """


class TeklinkConnection(SCPICmdRead):
    """The ``TEKLink:CONNection`` command.

    **Description:**
        - This command returns the current TekLink network connection.

    **Usage:**
        - Using the ``.query()`` method will send the ``TEKLink:CONNection?`` query.
        - Using the ``.verify(value)`` method will send the ``TEKLink:CONNection?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - TEKLink:CONNection?
    """


class Teklink(SCPICmdRead):
    """The ``TEKLink`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``TEKLink?`` query.
        - Using the ``.verify(value)`` method will send the ``TEKLink?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.connection``: The ``TEKLink:CONNection`` command.
        - ``.refclk``: The ``TEKLink:REFClk`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "TEKLink") -> None:
        super().__init__(device, cmd_syntax)
        self._connection = TeklinkConnection(device, f"{self._cmd_syntax}:CONNection")
        self._refclk = TeklinkRefclk(device, f"{self._cmd_syntax}:REFClk")

    @property
    def connection(self) -> TeklinkConnection:
        """Return the ``TEKLink:CONNection`` command.

        **Description:**
            - This command returns the current TekLink network connection.

        **Usage:**
            - Using the ``.query()`` method will send the ``TEKLink:CONNection?`` query.
            - Using the ``.verify(value)`` method will send the ``TEKLink:CONNection?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - TEKLink:CONNection?
        """
        return self._connection

    @property
    def refclk(self) -> TeklinkRefclk:
        """Return the ``TEKLink:REFClk`` command.

        **Description:**
            - This command sets or queries the current state of the intrument's TekLink Reference
              Output.

        **Usage:**
            - Using the ``.query()`` method will send the ``TEKLink:REFClk?`` query.
            - Using the ``.verify(value)`` method will send the ``TEKLink:REFClk?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TEKLink:REFClk value`` command.

        **SCPI Syntax:**

        ::

            - TEKLink:REFClk {ON|OFF|PENDING}
            - TEKLink:REFClk?

        **Info:**
            - ``ON`` specifies the on state of the TekLink Reference Output.
            - ``OFF`` specifies the off state of the TekLink Reference Output.
            - ``PENDING`` specifies the pending state of the TekLink Reference Output.
        """
        return self._refclk



# pylint: disable=line-too-long
"""The custom commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - CUSTOM:GATE<x>:SOUrce { CH1 | CH2 | CH3 | CH4 | MATH1 | MATH2 | MATH3 | MATH4| REF1 | REF2 | REF3 | REF4 }
    - CUSTOM:GATE<x>:START <NR3>
    - CUSTOM:GATE<x>:WIDth <NR3>
    - CUSTOM:SELECT:GATE<x> {1 | 0 | ON | OFF}
"""  # noqa: E501
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class CustomSelectGateItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``CUSTOM:SELECT:GATE<x>`` command.

    **Description:**
        - Sets or queries wether or not the selected gate is displayed on the screen.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``CUSTOM:SELECT:GATE<x> value`` command.

    **SCPI Syntax:**

    ::

        - CUSTOM:SELECT:GATE<x> {1 | 0 | ON | OFF}

    **Info:**
        - ``1`` sets the specified gate to be displayed.
        - ``0`` sets the specified gate to be hidden.
        - ``ON`` sets the specified gate to be displayed.
        - ``OFF`` sets the specified gate to be hidden.
    """


class CustomSelect(SCPICmdRead):
    """The ``CUSTOM:SELECT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``CUSTOM:SELECT?`` query.
        - Using the ``.verify(value)`` method will send the ``CUSTOM:SELECT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.gate``: The ``CUSTOM:SELECT:GATE<x>`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._gate: Dict[int, CustomSelectGateItem] = DefaultDictPassKeyToFactory(
            lambda x: CustomSelectGateItem(device, f"{self._cmd_syntax}:GATE{x}")
        )

    @property
    def gate(self) -> Dict[int, CustomSelectGateItem]:
        """Return the ``CUSTOM:SELECT:GATE<x>`` command.

        **Description:**
            - Sets or queries wether or not the selected gate is displayed on the screen.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``CUSTOM:SELECT:GATE<x> value``
              command.

        **SCPI Syntax:**

        ::

            - CUSTOM:SELECT:GATE<x> {1 | 0 | ON | OFF}

        **Info:**
            - ``1`` sets the specified gate to be displayed.
            - ``0`` sets the specified gate to be hidden.
            - ``ON`` sets the specified gate to be displayed.
            - ``OFF`` sets the specified gate to be hidden.
        """
        return self._gate


class CustomGateItemWidth(SCPICmdWrite):
    """The ``CUSTOM:GATE<x>:WIDth`` command.

    **Description:**
        - Sets or queries the width of the specified gate.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:WIDth value`` command.

    **SCPI Syntax:**

    ::

        - CUSTOM:GATE<x>:WIDth <NR3>
    """


class CustomGateItemStart(SCPICmdWrite):
    """The ``CUSTOM:GATE<x>:START`` command.

    **Description:**
        - Sets or queries the start position of the specified gate.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:START value`` command.

    **SCPI Syntax:**

    ::

        - CUSTOM:GATE<x>:START <NR3>
    """


class CustomGateItemSource(SCPICmdWrite):
    """The ``CUSTOM:GATE<x>:SOUrce`` command.

    **Description:**
        - Sets or queries the source for the specified gate.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - CUSTOM:GATE<x>:SOUrce { CH1 | CH2 | CH3 | CH4 | MATH1 | MATH2 | MATH3 | MATH4| REF1 | REF2 | REF3 | REF4 }
    """  # noqa: E501


class CustomGateItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``CUSTOM:GATE<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``CUSTOM:GATE<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``CUSTOM:GATE<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``CUSTOM:GATE<x>:SOUrce`` command.
        - ``.start``: The ``CUSTOM:GATE<x>:START`` command.
        - ``.width``: The ``CUSTOM:GATE<x>:WIDth`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._source = CustomGateItemSource(device, f"{self._cmd_syntax}:SOUrce")
        self._start = CustomGateItemStart(device, f"{self._cmd_syntax}:START")
        self._width = CustomGateItemWidth(device, f"{self._cmd_syntax}:WIDth")

    @property
    def source(self) -> CustomGateItemSource:
        """Return the ``CUSTOM:GATE<x>:SOUrce`` command.

        **Description:**
            - Sets or queries the source for the specified gate.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:SOUrce value``
              command.

        **SCPI Syntax:**

        ::

            - CUSTOM:GATE<x>:SOUrce { CH1 | CH2 | CH3 | CH4 | MATH1 | MATH2 | MATH3 | MATH4| REF1 | REF2 | REF3 | REF4 }
        """  # noqa: E501
        return self._source

    @property
    def start(self) -> CustomGateItemStart:
        """Return the ``CUSTOM:GATE<x>:START`` command.

        **Description:**
            - Sets or queries the start position of the specified gate.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:START value``
              command.

        **SCPI Syntax:**

        ::

            - CUSTOM:GATE<x>:START <NR3>
        """
        return self._start

    @property
    def width(self) -> CustomGateItemWidth:
        """Return the ``CUSTOM:GATE<x>:WIDth`` command.

        **Description:**
            - Sets or queries the width of the specified gate.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``CUSTOM:GATE<x>:WIDth value``
              command.

        **SCPI Syntax:**

        ::

            - CUSTOM:GATE<x>:WIDth <NR3>
        """
        return self._width


class Custom(SCPICmdRead):
    """The ``CUSTOM`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``CUSTOM?`` query.
        - Using the ``.verify(value)`` method will send the ``CUSTOM?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.gate``: The ``CUSTOM:GATE<x>`` command tree.
        - ``.select``: The ``CUSTOM:SELECT`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CUSTOM") -> None:
        super().__init__(device, cmd_syntax)
        self._gate: Dict[int, CustomGateItem] = DefaultDictPassKeyToFactory(
            lambda x: CustomGateItem(device, f"{self._cmd_syntax}:GATE{x}")
        )
        self._select = CustomSelect(device, f"{self._cmd_syntax}:SELECT")

    @property
    def gate(self) -> Dict[int, CustomGateItem]:
        """Return the ``CUSTOM:GATE<x>`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``CUSTOM:GATE<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``CUSTOM:GATE<x>?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.source``: The ``CUSTOM:GATE<x>:SOUrce`` command.
            - ``.start``: The ``CUSTOM:GATE<x>:START`` command.
            - ``.width``: The ``CUSTOM:GATE<x>:WIDth`` command.
        """
        return self._gate

    @property
    def select(self) -> CustomSelect:
        """Return the ``CUSTOM:SELECT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``CUSTOM:SELECT?`` query.
            - Using the ``.verify(value)`` method will send the ``CUSTOM:SELECT?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.gate``: The ``CUSTOM:SELECT:GATE<x>`` command.
        """
        return self._select



"""The save_and_recall commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - *SDS <NR1>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Sds(SCPICmdWrite):
    """The ``*SDS`` command.

    **Description:**
        - This command (no query form) changes the specified setup to reference the factory setup
          instead of the specific user setup slot. The content of the setup slot is unchanged, but
          the data will no longer be accessible to you. This command is equivalent to selecting
          Delete from the File menu, and then choosing the specified setup.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``*SDS value`` command.

    **SCPI Syntax:**

    ::

        - *SDS <NR1>

    **Info:**
        - ``<NR1>`` specifies a user setup location to delete. Setup storage location values range
          from 1 through 10; using an out-of-range value causes an error.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "*SDS") -> None:
        super().__init__(device, cmd_syntax)



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The format commands module.

These commands are used in the following models: SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


# pylint: disable=too-few-public-methods
class Format(BaseTSPCmd):
    """The ``format`` command tree.

    Constants:
        - ``.ASCII``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be ASCII format.
        - ``.BIGENDIAN``: Sets the binary byte order for the data that is printed using the
          printnumber() and
          printbuffer() functions to be most significant byte first.
        - ``.DREAL``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be double-precision IEEE Std 754 binary format.
        - ``.LITTLEENDIAN``: Sets the binary byte order for the data that is printed using the
          printnumber() and
          printbuffer() functions to be least significant byte first.
        - ``.NETWORK``: Sets the binary byte order for the data that is printed using the
          printnumber() and
          printbuffer() functions to be most significant byte first.
        - ``.NORMAL``: Sets the binary byte order for the data that is printed using the
          printnumber() and
          printbuffer() functions to be most significant byte first.
        - ``.REAL``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be double-precision IEEE Std 754 binary format.
        - ``.REAL32``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be single-precision IEEE Std 754 binary format.
        - ``.REAL64``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be double-precision IEEE Std 754 binary format.
        - ``.SREAL``: Sets the data format for data that is printed using the printnumber() and
          printbuffer() functions to be single-precision IEEE Std 754 binary format.
        - ``.SWAPPED``: Sets the binary byte order for the data that is printed using the
          printnumber() and
          printbuffer() functions to be least significant byte first.
    """

    ASCII = "format.ASCII"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be ASCII format."""
    BIGENDIAN = "format.BIGENDIAN"
    """str: Sets the binary byte order for the data that is printed using the printnumber() and
printbuffer() functions to be most significant byte first."""
    DREAL = "format.DREAL"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be double-precision IEEE Std 754 binary format."""
    LITTLEENDIAN = "format.LITTLEENDIAN"
    """str: Sets the binary byte order for the data that is printed using the printnumber() and
printbuffer() functions to be least significant byte first."""
    NETWORK = "format.NETWORK"
    """str: Sets the binary byte order for the data that is printed using the printnumber() and
printbuffer() functions to be most significant byte first."""
    NORMAL = "format.NORMAL"
    """str: Sets the binary byte order for the data that is printed using the printnumber() and
printbuffer() functions to be most significant byte first."""
    REAL = "format.REAL"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be double-precision IEEE Std 754 binary format."""
    REAL32 = "format.REAL32"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be single-precision IEEE Std 754 binary format."""
    REAL64 = "format.REAL64"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be double-precision IEEE Std 754 binary format."""
    SREAL = "format.SREAL"
    """str: Sets the data format for data that is printed using the printnumber() and
printbuffer() functions to be single-precision IEEE Std 754 binary format."""
    SWAPPED = "format.SWAPPED"
    """str: Sets the binary byte order for the data that is printed using the printnumber() and
printbuffer() functions to be least significant byte first."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "format") -> None:
        super().__init__(device, cmd_syntax)
